<?php
/**
 * $Horde: passwd/lib/Driver/smbldap.php,v 1.5 2005/06/23 20:36:57 selsky Exp $
 *
 * The LDAP class attempts to change a user's ldap password
 * and samba password stored in an
 * LDAP directory service.
 *
 * @author  Shane Boulter <sboulter@ariasolutions.com>
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @author  Tjeerd van der Zee <admin@xar.nl>
 * @author  Mattias Webjrn Eriksson <mattias@webjorn.org>
 * @author  Eric Jon Rostetter <eric.rostetter@physics.utexas.edu>
 * @package Passwd
 */
class Passwd_Driver_smbldap extends Passwd_Driver {

    /**
     * LDAP connection handle.
     *
     * @var resource
     */
    var $_ds;

    /**
     * Constructs a new Passwd_Driver_smbldap object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function Passwd_Driver_smbldap($params = array())
    {
        $this->_params['host']                = isset($params['host']) ? $params['host'] : 'localhost';
        $this->_params['sslhost']             = isset($params['sslhost']) ? $params['sslhost'] : '';
        $this->_params['port']                = isset($params['port']) ? $params['port'] : 389;
        $this->_params['encryption']          = isset($params['encryption']) ? $params['encryption'] : 'crypt';
        $this->_params['show_encryption']     = isset($params['show_encryption']) ? $params['show_encryption'] : 'true';
        $this->_params['uid']                 = isset($params['uid']) ? $params['uid'] : 'uid';
        $this->_params['basedn']              = isset($params['basedn']) ? $params['basedn'] : '';
        $this->_params['realm']               = isset($params['realm']) ? $params['realm'] : '';
        $this->_params['tls']                 = isset($params['tls']) ? $params['tls'] : null;
        $this->_params['attribute']           = isset($params['attribute']) ? $params['attribute'] : 'userPassword';
        $this->_params['lm_attribute']        = isset($params['lm_attribute']) ? $params['lm_attribute'] : 'sambaLMPassword';
        $this->_params['nt_attribute']        = isset($params['nt_attribute']) ? $params['nt_attribute'] : 'sambaNTPassword';
        $this->_params['pw_set_attribute']    = isset($params['pw_set_attribute']) ? $params['pw_set_attribute'] : 'sambaPwdLastSet';
        $this->_params['pw_expire_attribute'] = isset($params['pw_expire_attribute']) ? $params['pw_expire_attribute'] : 'sambaPwdMustChange';
        $this->_params['pw_expire_time']      = isset($params['pw_expire_time']) ? $params['pw_expire_time'] : 2147483647;
    }

    /**
     * Connects to an LDAP server and binds as the guest user or as the
     * optional userdn.
     *
     * @param string $userdn    The DN to use when binding non-anonymously.
     * @param string $password  The password for $userdn.
     * @param boolean $ssl      The SSL trigger.
     *
     * @return boolean  True on success, false otherwise.
     */
    function _connect($userdn = null, $password = null, $ssl = false)
    {
        if ($ssl && empty($this->_params['tls'])) {
            $this->_ds = ldap_connect('ldaps://' . $this->_params['sslhost']);
        } else {
            $this->_ds = ldap_connect($this->_params['host'], $this->_params['port']);
        }
        if (!$this->_ds) {
            return PEAR::raiseError(_("Could not connect to ldap server"));
        }

        if (ldap_set_option($this->_ds, LDAP_OPT_PROTOCOL_VERSION, 3) &&
            $this->_params['tls']) {
            if (!ldap_start_tls($this->_ds)) {
                return PEAR::raiseError(_("ldap_start_tls failed"));
            }
        }

        // Try to bind as the current userdn with password.
        if (!is_null($userdn)) {
            $result = @ldap_bind($this->_ds, $userdn, $password);
        }

        // If unable to bind as current user, still try to bind as
        // guest.
        if (empty($result)) {
            $result = @ldap_bind($this->_ds);
        }

        // If none of the bind attempts succeed, return an error.
        if (!$result) {
            return PEAR::raiseError(_("Could not bind to ldap server"));
        }

        return true;
    }

    /**
     * Lookup and return the user's dn.
     *
     * @param string $user     The username of the user.
     * @param string $passw    The password of the user.
     * @param string $realm    The realm (domain) name of the user.
     * @param string $basedn   The ldap basedn.
     * @param string $uid      The ldap uid.
     *
     * @return string  The ldap dn for the user.
     */
    function _lookupdn($user, $passw)
    {
        // Construct username@realm to connect as if 'realm' parameter
        // is set.
        $urealm = null;
        if (!empty($this->_params['realm'])) {
            $urealm = $user . '@' . $this->_params['realm'];
        }

        // Bind as current user. _connect will try as guest if no user
        // realm is found or auth error.
        $this->_connect($urealm, $passw);

        // Construct search.
        $search = $this->_params['uid'] . '=' . $user;

        // Get userdn.
        $result = ldap_search($this->_ds, $this->_params['basedn'], $search);
        $entry = ldap_first_entry($this->_ds, $result);
        if ($entry === false) {
            return PEAR::raiseError(_("User not found."));
        }
        $userdn = ldap_get_dn($this->_ds, $entry);

        return $userdn;
    }

    /**
     * Change the user's password.
     *
     * @param string $username      The user for which to change the password.
     * @param string $old_password  The old (current) user password.
     * @param string $new_password  The new user password to set.
     *
     * @return boolean  True or false based on success of the change.
     */
    function changePassword($username, $old_password, $new_password)
    {
        require_once 'Crypt/CHAP.php';

        // Get the user's dn.
        global $conf;

        if ($conf['hooks']['userdn']) {
            $userdn = Horde::callHook('_passwd_hook_userdn', array(Auth::getAuth()));
        } else {
            $userdn = $this->_lookupdn($username, $old_password);
            if (is_a($userdn, 'PEAR_Error')) {
                return $userdn;
            }
        }

        // Construct username@realm to connect if 'realm' parameter is
        // set.  Looks like the _passwd_hook_username hook, but here
        // we use a configurable parameter as realm.  This is a
        // safeguard that if unable to lookup the user's DN we still
        // have a chance to authenticate.
        if (empty($userdn) && !empty($this->_params['realm'])) {
            $userdn = $username . '@' . $this->_params['realm'];
        }

        // Connect as the user.
        $res = $this->_connect($userdn, $old_password, !empty($this->_params['sslhost']));
        if (is_a($res, 'PEAR_Error')) {
            if ($res->getMessage() == _("Could not bind to ldap server")) {
                return PEAR::raiseError(_("Incorrect Password"));
            }
            return $res;
        }

        // Change the user's password.
        $new_details[$this->_params['attribute']] = $this->encryptPassword($new_password);
        if (!ldap_mod_replace($this->_ds, $userdn, $new_details)) {
            return PEAR::raiseError(ldap_error($this->_ds));
        }

        $hash = &new Crypt_CHAP_MSv2();
        $hash->password = $new_password;
        $lmpasswd = strtoupper(bin2hex($hash->lmPasswordHash()));
        $ntpasswd = strtoupper(bin2hex($hash->ntPasswordHash()));
        $settime = time();
        // 24 hours/day * 60 min/hour * 60 secs/min = 86400 seconds/day
        $expiretime = $settime + ($this->_params['pw_expire_time'] * 86400);

        $new_lm_passwd[$this->_params['lm_attribute']] = $lmpasswd;
        if (!ldap_mod_replace($this->_ds, $userdn, $new_lm_passwd)) {
            return PEAR::raiseError(ldap_error($this->_ds));
        }

        $new_nt_passwd[$this->_params['nt_attribute']] = $ntpasswd;
        if (!ldap_mod_replace($this->_ds, $userdn, $new_nt_passwd)) {
            return PEAR::raiseError(ldap_error($this->_ds));
        }

        $new_set_time[$this->_params['pw_set_attribute']] = $settime;
        if (!ldap_mod_replace($this->_ds, $userdn, $new_set_time)) {
            return PEAR::raiseError(ldap_error($this->_ds));
        }

        $new_set_time[$this->_params['pw_expire_attribute']] = $expiretime;
        if (!ldap_mod_replace($this->_ds, $userdn, $new_set_time)) {
            return PEAR::raiseError(ldap_error($this->_ds));
        }

        return true;
    }

}
